@@ -1,3 +1,4 @@
+import numpy as np
 from copy import copy
 from eloquentarduino.ml.data.Dataset import Dataset
 from eloquentarduino.utils import jinja
@@ -53,6 +54,15 @@ class Pipeline:
 
         return step[0] if len(step) == 1 else None
 
+    @property
+    def dataset(self):
+        """
+        Convert pipeline data to Dataset object
+        """
+        assert self.y is None or len(self.y.shape) == 1 or self.y.shape[1] == 1, 'y MUST be None or 1d'
+
+        return Dataset(name=self.name, X=self.X, y=np.flatten(self.y), test_validity=False)
+
     @property
     def input_dim(self):
         """
@@ -113,13 +123,20 @@ class Pipeline:
 
         return Pipeline(self.name, Dataset('Dataset', self.X, self.y), steps)
 
-    def fit(self):
+    def fit(self, catch_error=False):
         """
         Fit the steps
+        :param catch_error: bool if True, returns None on common errors due to steps incompatibility
         :return: self
         """
         for step in self.steps:
-            self.X, self.y = step.fit(self.X, self.y)
+            try:
+                self.X, self.y = step.fit(self.X, self.y)
+            except ValueError as err:
+                if catch_error:
+                    return None
+                else:
+                    raise err
 
         return self
 
